import cv2 as cv
import numpy as np
import time
import serial
    
def displayImage(mat,title):
	# Show image with bounding bo
	cv.imshow(title,mat)

	k = cv.waitKey(0) & 0xFF
	print( 'Key with int code' + str(k) +' has been pressed')
	if k==ord('q'):
		print('User has pressed \'q\'; Closing all windows')
		cv.destroyAllWindows();
		cv.waitKey(1)
		cv.waitKey(1)
		cv.waitKey(1)
		cv.waitKey(1)

def findTriangleAngle(tri):
	a = tri[0,0,0:2];
	b = tri[1,0,0:2];
	c = tri[2,0,0:2];
	centroid = (a+b+c)/3
	d1 = np.linalg.norm(a-b)
	d2 = np.linalg.norm(c-b)
	d3 = np.linalg.norm(a-c)
	#print(d1)
	#print(d2)
	#print(d3)
	# FInd least distance
	if d2<d1:
		#print('d2 is less than d1')
		if d3<d2:
			#print('d3 is less than d2')
			mini = d3
		else:
			#print('d2 is less than d3')
			mini = d2
	else:
		#print('d1 is less than d2')
		if d3<d1:
			#print('d3 is less than d1')
			mini = d3
		else:
			#print('d1 is less than d3')
			mini = d1
	# Find which is the side		
	if mini==d1:
		mp = (a+b)/2
		op = c
	elif mini == d2:
		mp = (c+b)/2;	
		op = a
	else:
		mp = (a+c)/2;
		op = b
	return((mp,op,centroid))	
	



def getAngleDistance(frameCount):
	
	# Create new camera each time to avoid buffering
	camera = cv.VideoCapture(0)
	return_value, frColor = camera.read()
	del(camera);
	cv.imwrite(str(frameCount)+'.png', frColor)
	print('\n\n\n-------------------------------------')
	print('Frame ' + str(frameCount) + ' acquired from camera')
	# Blur the image
	frBlur = cv.GaussianBlur(frColor, (3, 3), 0)	
	#print('Frame ' + str(frameCount) + ' blurred')

	# Show effect of blur
	#displayImage(fr,'Blurred image')

	# Convert to grayscale
	frGray = cv.cvtColor(frBlur,cv.COLOR_BGR2GRAY);
	#print('Frame ' + str(frameCount) + ' converted to gray scale')

	# save fgray scale
	#cv.imwrite('fr_'+str(frameCount)+'grayScale.png',frGray)
	#print('Frame ' + str(frameCount) + ' written to hard disk')
	
	# APPLY THRESHOLD
	ret,frThresh = cv.threshold(frGray,210,255,0)

	# Show effect of threshold
	#displayImage(thresh,'Threshold of the image')

	# Find contour finder
	contours, hierarchy = cv.findContours(frThresh,cv.RETR_TREE,cv.CHAIN_APPROX_SIMPLE)
	#print('Frame ' + str(frameCount) + ' processed in contour finder')
		
	# Find center of contours
	count=0
	circCount = 0;
	rectangleCount=0;
	rect = np.zeros((4,10,2));
	for cont in contours:
		# Get approximate shape
		approx = cv.approxPolyDP(cont, 0.01*cv.arcLength(cont,True), True)
		
		# Draw approximate shape
		cv.drawContours(frColor, [approx], 0, (0), 5)
		x = approx.ravel()[0]
		y = approx.ravel()[1]
		if len(approx) == 3:
			cv.putText(frColor, "T", (x, y), ballusFont, 1, (255,255,255))
			print('Frame ' + str(frameCount) + ' found triangle')
			tri = approx;
		elif len(approx) == 4:
			cv.putText(frColor, "R", (x, y), ballusFont, 1, (255,255,255))
			print('Frame ' + str(frameCount) + ' found rectangle number '+str(rectangleCount))
			rect[:,rectangleCount,:] = approx.reshape(4,2);
			rectangleCount = rectangleCount+1;
		else:
			# Some times more than 1 circle gets detected	
			print('Frame ' + str(frameCount) + ' found circle')
			cv.putText(frColor, "C", (x, y), ballusFont, 1, (255,255,255))
			circ = approx;
		
		M = cv.moments(cont)
		if M["m00"] >0:
			cx = int(M["m10"]/M["m00"]);
			cy = int(M["m01"]/M["m00"]);
		else:
			cx = 0;
			cy = 0;
			
		# Draw the contours
		cv.drawContours(frColor, cont, -1, (0,255,0), 3)	
		
		# Mark the midpoints of the contour
		cv.circle(frColor, (cx, cy), 3, (255, 0, 0), -1)	
		cv.putText(frColor, str(count), (cx - 20, cy - 20),
			ballusFont, 0.5, (0, 0, 255), 2)
		# Increment contour count	
		count = count+1;
	# Draw detected contours
	cv.imwrite(str(frameCount)+'frColorContours.png',frColor)
	
	# Find the orientation of the triangle
	#print('Frame ' + str(frameCount) + ', searching for triangle')
	mp,op,triCentroid = findTriangleAngle(tri);

	# Draw the axis of triangle
	cv.line(frColor,tuple(op),tuple(mp),(0,0,255),2)

	# Find distance between pentagon and triangle
	print('Frame ' + str(frameCount) + ', finding distance between car and target')
	circCentroid_x = circ[:,0,0].mean()
	circCentroid_y = circ[:,0,1].mean()
	circCentroid = (int(circCentroid_x),int(circCentroid_y))
	dist = np.linalg.norm(triCentroid-circCentroid)	;
	cv.putText(frColor, 'Dist ='+str(int(dist)), (320,0+30),
			ballusFont, 0.5, (0, 255, 255), 1)
			
	# Find angle between triangle axis and circle
	print('Frame ' + str(frameCount) + ', finding angle between car and target')
	carVec = op-triCentroid;
	circVec = circCentroid - triCentroid;
	circVecMag = np.linalg.norm(circVec)
	carVecMag = np.linalg.norm(carVec)
	cosTheta = circVec.dot(carVec)/(circVecMag*carVecMag)
	theta = np.rad2deg(np.arccos(cosTheta))
	#cv.putText(frColor, 'Angle ='+str(int(theta)), (triCentroid[0],triCentroid[1]+45),
	#		ballusFont, 0.5, (0, 255, 255), 1)
	cv.putText(frColor, 'Angle ='+str(int(theta)), (320,0+45),
			ballusFont, 0.5, (0, 255, 255), 1)

	# Draw line for angle
	cv.line(frColor,circCentroid,tuple(triCentroid),(255,0,0),2)

	# Write output file
	cv.imwrite(str(frameCount)+'frColorProcessed.png',frColor)
	
	# Give back angle and distance
	#cv.imshow('image',frColor)
	return((theta,dist))
	
		

# Main script
#----------------	

# connect to serial port for transmitter
print('Attempting to connect to serial port ttyACM0')
ser = serial.Serial('/dev/ttyACM0',9600)  # open serial port
print('serial port name : ' + ser.name)
# Set my fonts
ballusFont = cv.FONT_HERSHEY_COMPLEX
#cv.namedWindow('image', cv.WINDOW_NORMAL)


# Align angle first
frameCount = 0;
maxFrames  = 10;
angle=10;

while ((angle >= 5) and (frameCount < maxFrames)):
	# Get current angle and distance.
	angle,distance = getAngleDistance(frameCount);
	print('Iter '+str(frameCount)+' Distance = ' + str(distance) + ', angle = '+str(angle) + ' degrees')
	if angle>0:
		print('Reducing angle, sending serial command d')
		ser.write('d');
	time.sleep(1);
	frameCount = frameCount+1

# wait for user to press enter
#raw_input('Press enter to continue::')
	
# Make distance zero	
maxFrames  = maxFrames*2;
distance = 1000;
while ((distance >= 100) and (frameCount<maxFrames)):
	# Get current angle and distance.
	angle,distance = getAngleDistance(frameCount);
	print('Reducing distance, sending serial command w')
	print('Iter '+str(frameCount)+' Distance = ' + str(distance) + ', angle = '+str(angle) + ' degrees')
	ser.write(b'w');
	time.sleep(1);
	frameCount = frameCount+1


# wait for user to press enter
print('\n\n---------------------Reached target')
raw_input('Press enter to continue::')
		
# End program
cv.destroyAllWindows()
cv.waitKey(1)
cv.waitKey(1)
cv.waitKey(1)
cv.waitKey(1)

print("End of program. Exiting :-)")
ser.close();
	
