#include <IRremote.h>

//set D10 as input signal pin for IR remote
int input_pin = 10;
IRrecv irrecv(input_pin);
decode_results signals;

// Setup motor drive pins
int leftFrwd = 3;
int rightFrwd = 5;
int frwdDrivingTime = 500;
int diffDrivingTime  = 500;

// LED for left right
int leftLED = 7;
int rightLED = 13;

// Run differential time for different intervals of time
//1 is 200 milliseconds
//2 is 400 milliseconds
//5 is 1 second

// Int speeds
int rightSpeedValue = 0;
int leftSpeedValue = 0;
int runStatus = 1;// frwd bkwd pause
void setup()
{
  pinMode(leftFrwd, OUTPUT);
  pinMode(leftLED, OUTPUT);

  pinMode(rightFrwd, OUTPUT);
  pinMode(rightLED, OUTPUT);

  Serial.begin(9600);
  Serial.println("Balaji, BC547 code + IR receiver");
  Serial.println("Starting serial out");

  irrecv.enableIRIn(); // enable input from IR receiver
  Serial.println("Enabled IR input");
}
void loop() {

  if (irrecv.decode(&signals)) {
    Serial.println("Received IR signal");
    Serial.println(signals.value);
    Serial.println(signals.value, HEX);
    irrecv.resume(); // get the next signal
    digitalWrite(rightLED, HIGH);
    delay(100);
    digitalWrite(rightLED, LOW);
    // Display current status
    Serial.print("Differential driving time = ");
    Serial.println(diffDrivingTime);
    Serial.print("Forward      driving time = ");
    Serial.println(frwdDrivingTime);

  }
  // 0  12582912
  // 1  12582913
  // 9  12582921
  // Right 12583003
  // Left 12583002
  // Front 12583000
  // Back 12583001
  // Blue 12583024
  // Yellow 12583023
  // Green 12583022
  // Red 12583021
  // Play Pause 12582948

  // Assign how long to go forward based settings in red, green, yellow and blue buttons
  if ((signals.value >= 12583021) && (signals.value <= 12583024)) {
    // 12583021 red is 1*0.5 seconds
    // 12583022 red is 2*0.5 seconds
    // 12583023 red is 3*0.5 seconds
    // 12583024 red is 4*0.5 seconds
    frwdDrivingTime = (signals.value  - 12583020) * 500;
  }

  // Assign how long to do differential drive based on number keys
  if ((signals.value >= 12582913) && (signals.value <= 12582921)) {
    // 1  12582913
    // 10 12582921
    diffDrivingTime = (signals.value - 12582912) * 200;
  }


  // Go straight on up button press
  if (signals.value == 12583000) {
    digitalWrite(rightFrwd, HIGH);
    digitalWrite(leftFrwd, HIGH);
    delay(frwdDrivingTime);
    digitalWrite(rightFrwd, LOW);
    digitalWrite(leftFrwd, LOW);
  }
  
  // STOP on play pause button press
  if (signals.value == 12582948) {
    digitalWrite(rightFrwd, LOW);
    digitalWrite(leftFrwd, LOW);
  }

  // Go right for a fixed time based on the diffDrivingTime delay set using numbers
  if (signals.value == 12583003) {
    digitalWrite(rightFrwd, HIGH);
    digitalWrite(leftFrwd, LOW);
    delay(diffDrivingTime);
    digitalWrite(rightFrwd, LOW);

  }
  // Go left for a fixed time based on the diffDrivingTime delay set using numbers
  if (signals.value == 12583002) {
    digitalWrite(rightFrwd, LOW);
    digitalWrite(leftFrwd, HIGH);
    delay(diffDrivingTime);
    digitalWrite(leftFrwd, LOW);

  }
  // Make signal.value=0 so that decision tree is clear during next iteration
  signals.value = 0;

}
